;*******************************************************************************
;    Filename:      pulsgen1509.asm                                            *
;    Author:        Vladimr Hejtmnek                                         *
;    Description:   Genertor impulz - programov obsluha                     *
;*******************************************************************************
;    Device 16F1509
;                   -----------U-----------  
;             +5V --|Vdd               Vss|-- GND
;           piezo --|RA5/oscin     GP0/PGD|-- DCLK
;  AD vstup - pot --|RA4/oscout    GP1/PGC|-- DDAT
;                 --|RA3/MCLR/Vpp      GP2|-- DSTR
;            LED9 --|RC5               RC0|-- level in - AD vstup
;                 --|RC4               RC1|-- NCO out
;    20,97152 MHz --|RC3               RC2|-- sync out
;      KB dek 1 --|RC6               RB4|-- KB sloupec 1
;      KB dek 2 --|RC7               RB5|-- KB sloupec 2
;      KB dek 3 --|RB7               RB6|-- KB sloupec 3
;                   -----------------------
; fosc = 4 MHz, 1 s = 1 000 000 tcy, 1 tcy ~ 1 s
; fosc = 31 kHz, 1 s = 7750 tcy, 1 tcy ~ 129 s
;*******************************************************************************
;                                                                              *
	list    p=16f1509			; list directive to define processor
;    LIST R=DEC
    errorlevel -302
	#include "p16f1509.inc"

    __CONFIG _CONFIG1, _CLKOUTEN_OFF & _MCLRE_OFF &_PWRTE_OFF & _WDTE_OFF & _FOSC_INTOSC
    __CONFIG _CONFIG2, _LVP_OFF & _STVREN_ON

;        _CONFIG1
;_FCMEN_OFF       _FCMEN_ON(*)   (bit 13)
;_IESO_OFF        _IESO_ON(*)    (bit 12)
;_CLKOUTEN_OFF(*) _CLKOUTEN_ON   (bit 11)
;_BOREN_NSLEEP    _BOREN_OFF   _BOREN_ON(*) _BOREN_SBODEN   (bit 9, 10)
;_CP_OFF(*)       _CP_ON         (bit 7)
;_MCLRE_OFF       _MCLRE_ON(*)   (bit 6)
;_PWRTE_OFF(*)    _PWRTE_ON      (bit 5)
;_WDTE_NSLEEP     _WDTE_OFF    _WDTE_ON(*)  _WDTE_SWDTEN    (bit 3, 4)
;_FOSC_ECH(*)     _FOSC_ECL    _FOSC_ECM    _FOSC_EXTRC   _FOSC_HS   _FOSC_INTOSC   _FOSC_LP   _FOSC_XT (0,1,2)
;
;        _CONFIG2
;_LVP_OFF        _LVP_ON(*)     (bit 13)
;_LPBOR_OFF(*)   _LPBOR_ON      (bit 11)
;_BORV_HI        _BORV_LO(*)    (bit 10)
;_STVREN_OFF     _STVREN_ON(*)  (bit 9)
;_WRT_ALL        _WRT_BOOT     _WRT_HALF    _WRT_OFF(*)  (bit 0, 1)

;(*) 0xffff value
;*******************************************************************************
;  Variable Definitions
;
disp0	equ	0x20	; jednotky na displeji
disp1	equ	0x21	; destky
disp2	equ	0x22	; sta
disp3	equ	0x23	; tisce
disp4	equ	0x24	; 10k na displeji
disp5	equ	0x25	; 100k na displeji
disp6	equ	0x26	; nezobrazuje se
disp7	equ	0x27	; nezobrazuje se
sdbuf	equ	0x28	; odtud se poslaj data do sriovho posuvnho registru (4094, HC595)
tmp 	equ	0x29	; promnn zpoovac smyky
avef	equ	0x2b	; star hodnota pro filtraci a vsledek (zbytek)
avel	equ	0x2c	; star hodnota pro filtraci a vsledek
aveh	equ	0x2d	; star hodnota pro filtraci a vsledek
frql	equ	0x2e	; 2. stupe filtrace kmitotu
frqh	equ	0x2f	; 2. stupe filtrace kmitotu
;
bin0	equ	0x30	;pomocn registr pi vpotech a konvezi bin2dec
bin1	equ	0x31	;pomocn registr pi vpotech a konvezi bin2dec
bin2	equ	0x32	;pomocn registr pi vpotech a konvezi bin2dec
bin3	equ	0x33	;pomocn registr pi vpotech
;cycles	equ	0x34	; promnn bin2dec
octcnt	equ	0x35	; promnn bin2dec
bitcnt	equ	0x36	; promnn bin2dec
deccnt	equ	0x37	; promnn bin2dec

ditmr	equ	0x38	; asova zobrazen rovn

swph	equ	0x39	; asova sweep
swpl	equ	0x3a	; asova sweep

avlf	equ	0x3b	; 1. filtr pi men rovn (zbytek)
avll	equ	0x3c	; 1. filtr pi men rovn
avlh	equ	0x3d	; 1. filtr pi men rovn
lvll	equ	0x3e	; 2. stupe filtrace men rovn
lvlh	equ	0x3f	; 2. stupe filtrace men rovn
;
;HEF buffer - ukldaj se registry 0x40 a 0x4f
fr1l	equ	0x40	;uloen kmitoet f1
fr1h	equ	0x41	;uloen kmitoet f1
fr1f	equ 0x42	;pznaky (rozsah) f1
fr2l	equ	0x43	;uloen kmitoet f2
fr2h	equ	0x44	;uloen kmitoet f2
fr2f	equ 0x45	;pznaky (rozsah) f2
fr3l	equ	0x46	;uloen kmitoet f3
fr3h	equ	0x47	;uloen kmitoet f3
fr3f	equ 0x48	;pznaky (rozsah) f3
fr4l	equ	0x49	;uloen kmitoet f4
fr4h	equ	0x4a	;uloen kmitoet f4
fr4f	equ 0x4b	;pznaky (rozsah) f4
; pamov msta 0x4c, 0x4d a 0x4e se tak ulo do HEF
ncohold	equ	0x4f	;pomocn registr pi zmn kmitotu modeset HI
;
fstrtl	equ	0x60	; start kmitoet pi vpotu jemnho ladn a rozmtn
fstrth	equ	0x61	; start kmitoet pi vpotu jemnho ladn a rozmtn
fstopl	equ	0x62	; stop kmitoet pi vpotu jemnho ladn a rozmtn
fstoph	equ	0x63	; stop kmitoet pi vpotu jemnho ladn a rozmtn
fgol	equ	0x64	; registr pro vpoet (fstopl) jemnho ladn a rozmtn
fgoh	equ	0x65	; registr pro vpoet (fstoph) jemnho ladn a rozmtn
kbscan	equ	0x66	; sejmut stav tlatek (kbread)
kbcode	equ	0x67	; sejmut stav tlatek (kbread2)
kbtmr	equ	0x68	; asova klvesnice (oeten zkmit)
svtmr	equ	0x69	; asova SAVE

stepl	equ	0x6a	; aktun krok pi jemnm ladn (= ave) a rozmtn
steph	equ	0x6b	; aktun krok pi jemnm ladn (= ave) a rozmtn

ledxch	equ 0x6c	; stav LED - posledn stav pro zjitn zmny
ledst	equ 0x6d	; stav LED - odtud se kopruj data pro zobrazen
;					  |x|x|x|x|x|x|x|x|
;					   | | | | | | | +- 1 - F4
;					   | | | | | | +--- 1 - F3
;					   | | | | | +----- 1 - F2
;					   | | | | +------- 1 - F1
;					   | | | +--------- 1 - sweep
;					   | | +----------- 1 - fine
;					   | +------------- 1 - man
;					   +--------------- 1 - high 10x f
	#define	SWEEP  	ledst,4		; pznak rozmtn mezi F3 a F4
	#define	FINE  	ledst,5		; pznak jemn ladn mezi F3 a F4
	#define	MANU	ledst,6		; pznak run ladn
	#define	R10X	ledst,7		; rozsah x 10
;
flags	equ	0x6e	; pznaky
;					  |x|x|x|x|x|x|x|x|
;					   | | | | | | | +- 1 - pznak zpornho sla
;					   | | | | | | +--- 1 - hrub ladn
;					   | | | | | +----- 1 - rozsah pi runm ladn MANR
;					   | | | | +------- 1 - save - do pamti bylo ukldno
;					   | | | +--------- 1 - F1
;					   | | +----------- 1 - F2
;					   | +------------- 1 - F3
;					   +--------------- 1 - F4
	#define	COAR	flags,1		; pznak hrub ladn
	#define	MANR	flags,2		; rozsah pi runm ladn
	#define	SAVE	flags,3		; pznak ukldalo se do pamti
	#define	F1  	flags,4		; pznak pamti F1
	#define	F2  	flags,5		; pznak pamti F2
	#define	F3  	flags,6		; pznak pamti F3
	#define	F4  	flags,7		; pznak pamti F4
;
flags2	equ	0x6f	; pznaky
;					  |x|x|x|x|x|x|x|x|
;					   | | | | | | | +- 1 - pznak vyho rozsahu
;					   | | | | | | +--- 1 - rozsah FINE - SWEEP pepotn
;					   | | | | | +----- 1 - men rovn
;					   | | | | +------- 1 - 
;					   | | | +--------- 1 - 
;					   | | +----------- 1 - 
;					   | +------------- 1 - 
;					   +--------------- 1 - 
	#define	R10F	flags2,0		; pznak vyho rozsahu
	#define	FISW	flags2,1		; rozsah FINE - SWEEP pepotn
	#define	LVLM	flags2,2		; men rovn
;	#define	SAVE	flags2,3		; pznak ukldalo se do pamti
;	#define	F1  	flags2,4		; pznak pamti F1
;	#define	F2  	flags2,5		; pznak pamti F2
;	#define	F3  	flags2,6		; pznak pamti F3
;	#define	F4  	flags2,7		; pznak pamti F4
;
ncol	equ	0x70	;pomocn registr pi zmn kmitotu NCO
ncoh	equ	0x71	;pomocn registr pi zmn kmitotu NCO
cycles	equ	0x72	; promnn bin2dec
hef_byte	equ	0x73
;
	#define	DCLK	PORTA,0		; sriov vstup - clk
	#define	DDAT	PORTA,1		; sriov vstup - data
	#define	DSTR	PORTA,2		; sriov vstup - pepis
	#define	SW10	PORTA,3		; vstup velk tlatko
	#define	BEEP	PORTA,5		; sriov vstup - piezo
	#define	SYNS	PORTC,0		; synchronizan impuls pi rozmtn
	#define	LED9	PORTC,5		; vstup na LED9 (nepouito)
;
;*******************************************************************************

	ORG 0x00
	goto start

;	ORG 0x04
;	retfie

;	ORG 0x0F
start

; init

; oscillator
	banksel	OSCCON	;(1)
;	movlw	0x00	;LF 31 kHz
;	movlw	0x68	;Fosc 4MHz_HF
	movlw	0x78	;IntOsc 16 MHz
	movwf	OSCCON
	clrf	OSCSTAT

; pin configuration
	banksel	LATA	;(2)
	clrf	LATA
	clrf	LATB
	clrf	LATC
	clrf	APFCON

	banksel	TRISA	;(1)
	movlw	0x18
	movwf	TRISA
	movlw	0x70	; bez SPI
	movwf	TRISB
	movlw	0x09	;vstup RC3 - NCO clk a RC0 - vstupn rove
	movwf	TRISC
;	bcf 	OPTION_REG,7	; povol WPU - u TMR0

	banksel	ANSELA	;(3)
	movlw	0x10
	movwf	ANSELA
	movlw	0x00	; digital IO
	movwf	ANSELB
	movlw	0x01	; RC0 vstup A/D
	movwf	ANSELC

	banksel	WPUA	;(4)
	movlw	0x08
	movwf	WPUA
	movlw	0x70
	movwf	WPUB

;Timer0 cofiguration
	banksel OPTION_REG
	movlw	0x57	; WPU-enable, Fosc/4 clock, prescaler 1:256
	movwf	OPTION_REG

; CLC configuration
	call	setdivide10

;NCO configuration
	banksel	NCO1CON
	movlw	0x40		; output en., pin en., low, 50% duty
	movwf	NCO1CON
;	movlw	0x03		; NCO clk pin RA5
;	movlw	0x01		; Fosc clk
	movlw	0x02		; CLC1 clk
	movwf	NCO1CLK
;	NCO1ACCU = 0x00;
;	NCO1ACCH = 0x00;
;	NCO1ACCL = 0x00;
	clrf	NCO1INCH
	clrf	NCO1INCL
	bsf 	NCO1CON,7	; enable NCO

;ADC configuration
	banksel	ADCON1		;
	movlw	0x0d		;b00001101 AN3 (RA4), AD enabled, stop
	movwf	ADCON0		;
	movlw	0xe0		; Right justify, fosc/64, Vref+ = Vdd (takt 16/20 MHz)(0xa0 pro fosc/32)
	movwf	ADCON1		;
	movlw	0x00		; No auto-conversion trigger selected
	movwf	ADCON2		;

;Interrupt
;	banksel - CORE
	movlw	0x00		; vechno zakzan
	movwf	INTCON

;inicializace promnnch
	banksel	PORTA
	clrf	ncohold	; vynuluj vy byte pi runm ladn (mon se bude tahat z FLASH
	clrf	aveh
	clrf	ledst
	bsf 	MANU	; vchoz - run ladn
	clrf	flags	; nuluj pznaky
	clrf	flags2	; nuluj pznaky
hefhef
	call	hefread	; ulo data z FLASH do RAM (159 tcy i s call a return) (Cina 432 tcy)
;
; main program
loop
	call	adconvert	; vrac v frqh:frql vyfiltrovan nastaven kmitotu
						; a v lvlh:lvll vyfiltrovan men rovn z AD pevodnku (464 tcy)
	call	kbread		; vrac v kbcode slo stisknutho tlata (1 - 9, 0 dn) (40 tcy)
	call	command		; nastav funkci zvolenou tlatky (11 tcy)
	call	removeflag	; zru pznak FISW pi zmn funkce

	btfsc	MANU		; run ladn?
	call	modeset		;  ano - nastaven kmitotu hrub (pi stisku tlatka) a jemn (14 tcy)
	btfsc	FINE		; ladn mezi F3 a F4?
	call	fiswtune	;  ano
	btfsc	SWEEP		; rozmtn mezi F3 a F4?
	call	fiswtune	;  ano - zatek podprogramu je stejn pro FINE i SWEEP

	call	range		; nastav dliku podle rozsahu
	call	setnco		; zape data do NCO

	btfsc 	LVLM
	goto	displevel	; zobrazen rovn msto kmitotu, ve ostatn beze zmny

	call	disprange	; pepot data pro zobrazen displeje podle rozsahu ( zatm 96 nebo 101 tcy)
	call	bin2dec		; (1243 tcy)
	call	removezeros	; (25 tcy)
	call	wr_serial	; (528 tcy)
	goto	loop	;loop forever

displevel
	decfsz	ditmr,f		; zmen obsah asovacho registru
	goto	mult5
	bcf 	LVLM		; zru pznak men napt
mult5
; vynsobit 5 (1 bit ~ 5 mV a zapsat do bin0 a bin1 (bin2 = 0)
	bcf		STATUS,C
	rlf 	lvll,w		;avll,w
	movwf	bin0
	rlf 	lvlh,w
	movwf	bin1
	rlf 	bin0,f
	rlf 	bin1,f
	movf	lvll,w		;avll,w
	addwf	bin0,f
	movf	lvlh,w
	addwfc	bin1,f
; konverze na BCD
	clrf	bin2		; res3
	call	bin2dec
; upravit zobrazen displeje
	movlw	.10 	; znak " "
	movwf	disp4
	movlw	.11 	; znak "u"
	movwf	disp5
; zapi na displej (a LED)
	call	wr_serial	; (528 tcy)
	goto	loop	;loop forever

;---------------- podprogramy na page 0 -----------------

znak	;poad bit |DT|c|d|e|b|a|f|g| - (posuv. reg. s 4094)
		;poslat od nejvyho bitu, segment svt na log.0
		;podprogram umstn na page 0, jinak by se musel nastavit PCLATH
;	clrf	PCLATH	;page 0
	andlw	0x0f		;jen spodn 4 bity
	addwf	PCL,f
	retlw	0x81	;07eh	; znak 0
	retlw	0xb7	;048h	; znak 1
	retlw	0xc2	;03dh	; znak 2
	retlw	0x92	;06dh	; znak 3
	retlw	0xb4	;04bh	; znak 4
	retlw	0x98	;067h	; znak 5
	retlw	0x88	;077h	; znak 6
	retlw	0xb3	;04ch	; znak 7
	retlw	0x80	;07fh	; znak 8
	retlw	0x90	;06fh	; znak 9
	retlw	0xff	;000h	; znak " "
	retlw	0x8f	;070h	; znak "u" b-1000 1111
	retlw	0xe0	;01fh	; znak "P"
	retlw	0x8c	;073h	; znak "b"
	retlw	0xee	;011h	; znak "r"
	retlw	0xfe	;001h	; znak "-"

;	retlw	0xcd	;032h	; znak "L"
;	retlw	0xa4	;05bh	; znak "H"
;	retlw	0xef	;010h	; znak "i"
;	retlw	0x8e	;071h	; znak "o"
;	retlw	0xcc	;033h	; znak "t"
;	retlw	0xd2	;02dh	; znak "stupe"

;	retlw	0xa0	;05fh	; znak "A"
;	retlw	0x8c	;073h	; znak "b"
;	retlw	0xce	;031h	; znak "c"
;	retlw	0x86	;079h	; znak "d"
;	retlw	0xc8	;037h	; znak "E"
;	retlw	0xe8	;017h	; znak "F"

;--------------------------------------------------------------------------
command
;	movlw	0x01	; kdy program je na page 1
;	movwf	PCLATH
	movf	kbcode,w	; nati povel
	andlw	0x0f	; jen spodn 4 bity
	addwf	PCL,f

	goto	comm2	; 00h - dn stisknut tlatko z pole 3x3
	goto	mem4	; 01h - kmitoet 4
	goto	mem3	; 02h - kmitoet 3
	goto	mem2	; 03h - kmitoet 2
	goto	mem1	; 04h - kmitoet 1
	goto	sweep	; 05h - funkce SWEEP
	goto	finet	; 06h - jemn ladn mezi f3 a F4
	goto	manual	; 07h - ladn v celm rozsahu
	goto	hilo	; 08h - rozsah 65/655 kHz
	goto	outlvl	; 09h - men vstupnho napt
comm2
	btfss	SW10
	goto	coarse	; hrub ladn
	goto	nosw	; 00h - dn stisknut tlatko
;--------------------------------------------------------------------------
wr_serial	; zape data do sriovho posuvnho registru
;displej
	movf	disp0,w
	call	znak		; kter segmenty?
	call	wr_byte
	movf	disp1,w
	call	znak		; kter segmenty?
	call	wr_byte
	movf	disp2,w
	call	znak		; kter segmenty?
	call	wr_byte
	movf	disp3,w
	call	znak		; kter segmenty?
	btfsc 	LVLM		; desetinn teka pi men rovn
	andlw	0x7f
	call	wr_byte
	movf	disp4,w
	call	znak		; kter segmenty?
	call	wr_byte
	movf	disp5,w
	call	znak		; kter segmenty?
	call	wr_byte
	movf	ledst,w
	call	wr_byte
	bsf		DSTR		; zobraz
	bcf		DSTR
	return

wr_byte		; data na zapsn do 4094 ve WREG
			; Q1 nejni, Q8 nejvy bit
			; WREG data |DT|c|d|e|b|a|f|g| segment displeje
	movwf	sdbuf
	bcf		DCLK
	movlw	.8
	movwf	tmp
write_loop
	rlf		sdbuf,f		;pesu nejvy bit do carry
	bcf		DDAT
	btfsc	STATUS,C	;test C
	bsf		DDAT		;kdy C=1, zapi 1, segment nesvt
	bsf		DCLK		;zapisovac impuls (toggle DCLK)
	bcf		DDAT		;nulovn vstupu kvli sdlen vvodu
	bcf		DCLK
	decfsz	tmp,f		;smyka 8x (loop 8 times)
	goto	write_loop
;	rlf		sdbuf,w		; vrt pvodn obsah do WREG
	return
;--------------------------------------------------------------------------
removezeros
	movlw	.10			; znak " "
	movf	disp5,f		; nastav pnak ZERO
	btfss	STATUS,Z
	return
	movwf	disp5

	movf	disp4,f		; nastav pnak ZERO
	btfss	STATUS,Z
	return
	movwf	disp4

	movf	disp3,f		; nastav pnak ZERO
	btfss	STATUS,Z
	return
	movwf	disp3

	movf	disp2,f		; nastav pnak ZERO
	btfss	STATUS,Z
	return
	movwf	disp2

	movf	disp1,f		; nastav pnak ZERO
	btfss	STATUS,Z
	return
	movwf	disp1
	return
;--------------------------------------------------------------------------
;SHORTER BCD CONVERSION (Michael McLoughlin - April '01)
;I have modified Steve Teal?s code, which required 1957 cycles, so that it completes in 1242 cycles!
;Steve?s code doubles his Travelling Total (TT), but this only grows slowly and initially only one digit is needed
; to handle it. Yet the subroutine always doubles the whole of TT, so almost half the RLF multiplications
; do 2 ? 0 + 0 = 0, and are superfluous. By studying the worst case (all 24 bits = 1) it soon appears that we only
; need to involve a new decimal digit for every three binary digits. The 08 in Steve?s listing could be called cycles,
; to start at 01 and increment after every three bits. Another counter (octcnt) ensures the repetition of that whole
; procedure just eight times.
;In the following listing the commands to delete are shown "remmed out" with a semicolon, and the new lines are
; notated as such. +
;http://www.piclist.com/techref/microchip/math/radix/b2bu-24b9d-xx.htm
bin2dec:
	clrf	FSR0H
	clrf	disp0
	clrf	disp1
	clrf	disp2
	clrf	disp3
	clrf	disp4
	clrf	disp5
	clrf	disp6
	clrf	disp7
;   movlw 18	 		; deleted
	clrf	cycles		; new
	movlw	.8			; new
	movwf	octcnt		; new
octloop:				; new
	incf	cycles,f	; new
	movlw	.3			; new
	movwf	bitcnt
bitloop:
	rlf 	bin0,f
	rlf 	bin1,f
	rlf 	bin2,f
	movlw	disp0
	movwf	FSR0L
;   movlw 08			; deleted
	movfw	cycles		; new
	movwf	deccnt
decloop:
	rlf 	INDF0,f
	movlw	0xF6
	addwf	INDF0,0
	btfsc	STATUS,C
	movwf	INDF0
	incf	FSR0L,f
	decfsz	deccnt,f
	goto	decloop
	decfsz	bitcnt,f
	goto	bitloop
	decfsz	octcnt,f	; new
	goto	octloop		; new
    return

;--------------------------------------------------------------------------
adconvert	; vrac v frqh:frql nastaven kmitotu potenciometrem a v lvlh:lvll zmenou rove na vstupu
; promnn aveh:avel:avef , frqh:frql , avlh:avll:avlf a lvlh:lvll se nesmj do ptho men mnit!
	banksel	ADCON0
	bsf 	ADCON0,GO		;start AD pevodu - trv 48 Tcy
	btfsc	ADCON0,GO		;done/hotovo?
	goto	$-1
;	clrwdt
	movf	ADRESL,w		;uloen niho byte do W
	banksel	PORTA
	movwf	bin1
	banksel	ADCON0
	movf	ADRESH,w
	banksel	PORTA
	movwf	bin2
; nastav AD pro men rovn
	banksel	ADCON0
	movlw	0x11		;nastav AN4 (RC0), AD enabled, stop
	movwf	ADCON0		;
	banksel PORTA
;
; digitln filtrace
;		NEW = bin2:bin1 - nov hodnota ped filtrac
;		AVE = aveh:avel:avef - star hodnota, pak nov hodnota po filtraci
;
;         (WIDTH-1)*AVE + NEW			AVEHI  Holds the average [0-255].
;   AVE = -------------------			AVELO  Holds the fractional part of the average.
;                WIDTH       			NEW    Holds the new sample.
;
;                                         NEW - AVE
; upraven vzorec pro vpoet AVE = AVE + ---------
;                                            256
	clrf	bin3
	clrf	bin0		; 'NEW' = NEW - AVE
	comf	bin0,f

	movf	avef,w		; b = b - a; NEW = b; AVE = a
	subwf	bin0,f
	movf	avel,w
	btfss	STATUS,C
	incfsz	avel,w
	subwf	bin1,f
	movf	aveh,w
	btfss	STATUS,C
	incfsz	aveh,w
	subwf	bin2,f

	btfss	STATUS,C
	comf	bin3,f

; vpoet AVE = AVE + 'NEW'/256   b = b + a
	movf	bin1,w		; bin1/256 je na pozici bin0
	addwf	avef,f		; average fragment
	movf	bin2,w		; bin2/256 je na pozici bin1
	btfsc	STATUS,C
	incfsz	bin2,w
	addwf	avel,f		;
	movf	bin3,w		; MS byte
	btfsc	STATUS,C
	incfsz	bin3,w
	addwf	aveh,f		;

; 2. stupe digitln filtrace
;compare_unsigned_16: ; 7
	movf	aveh,w		;Nhi
	subwf	frqh,w		;tmfh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	fi2fhi		;ne
	movf	avel,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	frql,w		;tmfl,w		;Plo, subtract P-X
fi2fhi
; if N=P then now Z=1.
; if N>P then now C=0.
; if N<=P then now C=1.

; Filtrace (N - nov hodnota, P - hodnota pro displej
;    N <= P -> P = N		(Z=0, C=1 - zkopruj)
;    N > P  -> P = N-1		(Z=0, C=0 - odeti 1 a zkopruj)
	btfsc	STATUS,C	;
	goto	fi2copy		;jen zkoprovn
; -1 a zkoprovn
	movlw	.1			;
	subwf	avel,w		;
	movwf	frql		;
	movf	aveh,w		;
	btfss	STATUS,C	;
	decf	aveh,w		;
	movwf	frqh		;
	goto	adlevel
fi2copy	 ;zkoprovn
	movf	aveh,w		;
	movwf	frqh		;
	movf	avel,w
	movwf	frql		;

adlevel
; naten napt z kondenztoru vstupnho budie
	banksel	ADCON0
	bsf 	ADCON0,GO	;start AD pevodu - trv 48 Tcy
	btfsc	ADCON0,GO	;done/hotovo?
	goto	$-1
;	clrwdt
	movf	ADRESL,w	;uloen niho byte do W
	banksel	PORTA
	movwf	bin1		;lvll
	banksel	ADCON0
	movf	ADRESH,w
	banksel	PORTA
	movwf	bin2		;lvlh
;nastav zptky AN3 (RA4) - nastaven kmitotu
	banksel	ADCON0
	movlw	0x0d		
	movwf	ADCON0		;
	banksel	PORTA
;
; digitln filtrace
;		NEW = bin2:bin1 - nov hodnota ped filtrac
;		AVE = avlh:avll:avlf - star hodnota, pak nov hodnota po filtraci
;
;                                         NEW - AVE			AVEHI  Holds the average [0-255].
; upraven vzorec pro vpoet AVE = AVE + ---------			AVELO  Holds the fractional part of the average.
;                                            256			NEW    Holds the new sample.
	clrf	bin3
	clrf	bin0		; 'NEW' = NEW - AVE
	comf	bin0,f
	movf	avlf,w		; b = b - a; NEW = b; AVE = a
	subwf	bin0,f
	movf	avll,w
	btfss	STATUS,C
	incfsz	avll,w
	subwf	bin1,f
	movf	avlh,w
	btfss	STATUS,C
	incfsz	avlh,w
	subwf	bin2,f
	btfss	STATUS,C
	comf	bin3,f
;
; vpoet AVE = AVE + 'NEW'/256   b = b + a
;
	movf	bin1,w		; bin1/256 je na pozici bin0
	addwf	avlf,f		; average fragment
	movf	bin2,w		; bin2/256 je na pozici bin1
	btfsc	STATUS,C
	incfsz	bin2,w
	addwf	avll,f		;
	movf	bin3,w		; MS byte
	btfsc	STATUS,C
	incfsz	bin3,w
	addwf	avlh,f		;
;
; 2. stupe digitln filtrace
;compare_unsigned_16
	movf	avlh,w		;Nhi
	subwf	lvlh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi	;ne
	movf	avll,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	lvll,w		;Plo, subtract P-X
resulthi
; if N=P then now Z=1.
; if N>P then now C=0.
; if N<=P then now C=1.
; Filtrace (N - nov hodnota, P - hodnota pro displej
;    N <= P -> P = N		(Z=0, C=1 - zkopruj)
;    N > P  -> P = N-1		(Z=0, C=0 - odeti 1 a zkopruj)
	btfsc	STATUS,C	;
	goto	rescopy		;
	movlw	.1			;
	subwf	avll,w		;
	movwf	lvll		;
	movf	avlh,w		;
	btfss	STATUS,C	;
	decf	avlh,w		;
	movwf	lvlh		;
	return
rescopy
	movf	avlh,w		;
	movwf	lvlh
	movf	avll,w
	movwf	lvll
	return
;--------------------------------------------------------------------------
modeset	; nastaven kmitotu hrub (pi stisku tlatka) a jemn
		; vstup - frqh:frql (aveh:avel) (nemn se)
		; vstup - ncoh:ncol, ncohold (pam vyho byte)

;test
;	movlw	.85
;	movwf	ncol
;	movlw	.3
;	movwf	ncoh
;	bcf 	STATUS,C
;/test

	banksel	PORTA
	btfss	COAR		; je stisknuto tlatko?
	goto	modetst2	; nen
	rrf 	frqh,w		;aveh,w	;ncoh,f	;pesun nejvych 8 bit 10bitovho sla do niho byte
	movwf	ncol
	rrf		frql,w		;avel,w	;ncol,f
	movwf	ncoh
;	rrf 	ncoh,f	;pesun nejvych 8 bit 10bitovho sla do niho byte
;	rrf		ncol,f
	rrf 	ncol,f	;ncoh,f
	rrf		ncoh,f	;ncol,f
	movf	ncoh,w
	movwf	ncohold		; ulo novou hodnotu hrubho ladn
	clrf	ncol
	return
modetst2	
	movf	frql,w		;avel,w
	movwf	ncol
	movf	ncohold,w		;seti s vym bajtem
	addwf	frqh,w		;aveh,w	;ncoh,f
	movwf	ncoh
	btfss 	STATUS,C	; peteklo to?
	return				; ne
	movlw	0xff		; ano, nastav maximum
	movwf	ncoh
	movwf	ncol
	return

;--------------------------------------------------------------------------
wait
;Delay 197633 cycles
	movlw	.0
	movwf	cycles
	movlw	.0
	movwf	tmp
	decfsz	tmp,f
	goto	$-1
	decfsz	cycles,f
	goto	$-5
;End of Delay
	return

;--------------------------------------------------------------------------
fiswtune	; jemn ladn od f3 do f4 pi stisku TL6
			; rozmtn od f3 do f4 pi stisku TL5
			; vstup - aveh:avel, fr3h:fr3l, fr4h:fr4l
			; vstup - ncoh:ncol
			; pomocn - fstarth:fstartl, fstoph:fstopl

; kontrola konzistence rozsah
	btfsc	FISW		; rozsah pepotn?
	goto	setfr		;  ano
;
; mus bt pekoprovno do jinch registr kvli pepotvn rozsahu, pokud F3 m jin rozsah ne F4
	movf	fr3l,w		
	movwf	fstrtl		; nati f3
	movf	fr3h,w
	movwf	fstrth		; nati f4
	movf	fr4l,w
	movwf	fstopl		; nati f4
	movf	fr4h,w
	movwf	fstoph		; nati f4
;
	btfsc	fr3f,0		; rozsah f3 HI?
	goto	fiswhi		;  ano
	btfsc	fr4f,0		;  ne - test rozsah f4 HI?
	goto	errLoHi		;  ano - chyba
	bcf 	R10X		;  ne - rozsah LO - zhasni LED8
	goto	setfr-1
fiswhi
	btfss	fr4f,0		; rozsah f4 HI?
	goto	errHiLo		;  ne - chyba
setfrhi
	bsf 	R10X		;  ano - rozsah HI - rozsvi LED8
	bsf 	FISW		; nastav pznak rozsah pepotn
setfr
	movf	fstopl,w
	movwf	fgol		; registr pro vpoet
	movf	fstoph,w
	movwf	fgoh		; registr pro vpoet

stepfr	; aktuln krok = (freq2 - freq1)*step/1024 + freq1
; freq1: prvn, obvykle ni kmitoet
; freq2: druh, obvykle vy kmitoet
; frqh:frql (aveh:avel): krok ladn - vstup AD (ledn potenciometrem) nebo krok rozmtn (sweep)
; vsledek vpotu je v ncoh:ncol

; 16-bit Subtraction-with-Borrow
;       fstrth:fstrtl - slo, kter bude odetno (Number to be subtracted)
;       fgoh:fgol - slo, od kterho se odet (Number to be subtracted FROM)
;Out    fgoh:fgol - vsledek (Result)
;       Carry - NOT( Borrow result)
	movf	fstrtl,w
	subwf	fgol,f
	movf    fstrth,w
	btfss	STATUS,C
	incfsz  fstrth,w
	subwf	fgoh,f		;nco = nco - suma, WITH VALID CARRY, the Z flag is not valid.
; v fgoh:fgol je rozdl fgo-fstart

	bcf 	flags,0		; nuluj pznak zpornho sla
	btfsc	STATUS,C	;bylo freq1>freq2 ?
	goto	stepset	;  ne
						;  ano 
; zporn slo je teba pevst na kladn, aby bylo mon pout unsigned nsoben
; po vpotu kroku je teba slo opt pevst na zporn
	bsf 	flags,0		; nastav pznak zpornho sla
	comf	fgol,f	; dvojkov doplnk
	comf	fgoh,f
	incf	fgol,f
	btfsc   STATUS,Z	; je 0 -> peteklo to
	incf	fgoh,f

stepset
;tady je v fgoh:fgol rozdl kmitot tj. krok * 1024
; aktuln krok je bu nastaven potenciometrem - frqh:frql (aveh:avel) z AD pevodu - pro jemn ladn (FINE)
; nebo vpotem u rozmtn (SWEEP)
	bcf 	SYNS		; nuluj vstup se synchronizanm impulsem
	btfsc	FINE		; jemn ladn mezi F3 a F4?
	goto	finestep	;  ano

; aktun krok SWEEP
	decfsz	swpl,f		; asova rychlosti rozmtn
	return				; nedobhl - zatm se nic nedje
;swpl = 0 ; naplnit swpl podle potenciometru (frqh:frql) a pokraovat na nextstep
	rrf 	frqh,w	; chceme jen hornch 8 bit z 10bitov hodnoty frqh:frql
	movwf	swph
	rrf 	frql,w	;
	movwf	swpl
	rrf 	swph,f	;
	rrf 	swpl,f	;
	incf	swpl	; tady je tch 8 bit, co potebujeme

	incf	stepl,f
	btfss   STATUS,Z	; je 0 -> peteklo to
	goto	nextstep	;  ne - jdi dl
	incf	steph,f
	movlw	0x03
	andwf	steph,f

	btfsc   STATUS,Z	; je 0 -> zatek novho cyklu
	bsf 	SYNS		;  ano - vygeneruj synchronizan impuls
	goto	nextstep	;  (ne) - jdi dl

finestep ; aktuln krok je uren potenciometrem
	movf	frql,w	;avel,w
	movwf	stepl
	movf	frqh,w	;aveh,w
	movwf	steph

nextstep ; unsigned nsoben |fgoh:fgol| * step
;malin@onspec.co.uk   http://www.piclist.com/techref/microchip/math/mul/16x16umalin.htm
;unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
;	Program length	32 line 	time 129 to 228 cycles
;
;res -> bin     a2:a1 -> fgoh:fgol   b2:b1 -> steph:stepl (nemn se)
;
	clrf	bin3	; res4
	clrf	bin2	; res3
	clrf	bin1	; res2
	movlw	0x80
	movwf	bin0	; res1
nextbit
	rrf 	fgoh,f	; a2,f
	rrf 	fgol,f	; a1,f
	btfss	STATUS,C
	goto	nobit_l
;	movf	avel,w	;
	movf	stepl,w		; b1,w
	addwf	bin1,f		; res2,f
;	movf	aveh,w	;
	movf	steph,w		; b2, w
	btfsc	STATUS,C
;	incfsz	aveh,w	;
	incfsz	steph,w		; b2, w
	addwf	bin2,f		; res3, f
	btfsc	STATUS,C
	incf	bin3,f		; res4, f
	bcf		STATUS,C
nobit_l
	btfss	fgol,7	; a1, 7
	goto	nobit_h
;	movf	avel,w	;
	movf	stepl,w		; b1,w
	addwf	bin2,f		; res3,f
;	movf	aveh,w	;
	movf	steph,w		; b2, w
	btfsc	STATUS,C
;	incfsz	aveh,w	;
	incfsz	steph,w		; b2, w
	addwf	bin3,f		; res4, f
nobit_h
	rrf		bin3,f		; res4,f
	rrf		bin2,f		; res3,f
	rrf		bin1,f		; res2,f
	rrf		bin0,f		; res1,f
	btfss   STATUS,C
	goto	nextbit

; te vydlit 1024 -> nejni byte se zahod (bin0) + 2x rotace vpravo
	rrf		bin3,f		; res4,f
	rrf		bin2,f		; res3,f
	rrf		bin1,f		; res2,f
	rrf		bin3,f		; res4,f
	rrf		bin2,f		; res3,f
	rrf		bin1,f		; res2,f
; vsledek bin2:bin1, nejvy byte (bin3) by ml bt 0 (na nejvych 2 bitech narotovny data C+nejni bit bin1)
; zaokrouhlen
	btfss   STATUS,C
	goto	setnegative	; C=O, nebude se zaokrouhlovat
	incf	bin1,f		; zaokrouhl nahoru
	btfsc   STATUS,Z	; je 0 -> peteklo to
	incf	bin2,f
; peveden na zporn slo, pokud byl rozdl kmitot zporn
setnegative
	btfss   flags,0
	goto	addfreq		; =O, nebude se pevdt na zporn slo
	comf	bin1,f	; dvojkov doplnk
	comf	bin2,f
	incf	bin1,f
	btfsc   STATUS,Z	; je 0 -> peteklo to
	incf	bin2,f

addfreq ; sete bin2:bin1 + freq1h:freq1l
	movf	fstrtl,w	; stn s uloenm jinam
	addwf	bin1,w
	movwf	ncol
	movf	fstrth,w
	btfsc   STATUS,C
;	incfsz	fstrth,w
	incf	fstrth,w
	addwf	bin2,w
	movwf	ncoh
	return
;	goto	loop1

;---------
errLoHi		;  chyba f3 = Lo, f4 = Hi
; pepoti fstart = fstart/10
	movf	fstrtl,w
	movwf	bin0
	movf	fstrth,w
	movwf	bin1
	call	divide10
	movf	bin0,w
	movwf	fstrtl
	movf	bin1,w
	movwf	fstrth
	call	beep
;	bsf 	FISW	; nastav pznak rozsah pepotn
	goto	setfrhi
errHiLo		;  chyba f3 = Hi, f4 = Lo
; pepoti fstop = fstop/10
	movf	fstopl,w
	movwf	bin0
	movf	fstoph,w
	movwf	bin1
	call	divide10
	movf	bin0,w
	movwf	fstopl
	movf	bin1,w
	movwf	fstoph
	call	beep
;	bsf 	FISW	; nastav pznak rozsah pepotn
	goto	setfrhi

;--------------------------------------------------------------------------
kbread
;ten dku 1
	clrw
	banksel	TRISA
	bsf 	TRISC,7
	bsf 	TRISB,7
	banksel	PORTA
	btfss	PORTB,4
	movlw	.1
	btfss	PORTB,5
	movlw	.4
	btfss	PORTB,6
	movlw	.7
;ten dku 2
	banksel	TRISA
	bsf 	TRISC,6
	bcf 	TRISC,7
	banksel	PORTA
	btfss	PORTB,4
	movlw	.2
	btfss	PORTB,5
	movlw	.5
	btfss	PORTB,6
	movlw	.8
;ten dku 3
	banksel	TRISA
	bsf 	TRISC,7
	bcf 	TRISB,7
	banksel	PORTA
	btfss	PORTB,4
	movlw	.3
	btfss	PORTB,5
	movlw	.6
	btfss	PORTB,6
	movlw	.9
;vchoz stav
	banksel	TRISA
	bcf 	TRISC,6
	bcf 	TRISC,7
	banksel	PORTA
;testovac vstup
;	clrw
;/test
	movwf	kbcode
	return

;--------------------------------------------------------------------------
mem4	; 01h - kmitoet 4
	btfsc	SAVE	; bylo u uloeno?
	return			;  ano, tak u nic nen teba dlat
	bsf 	F4  	;  ne, nastav pznak a pokrauj
	banksel	OPTION_REG
	btfsc 	OPTION_REG,T0CS	;(1)je asova sputn?
	goto	memtmrset			;  nen
	banksel	PORTA
	btfss 	INTCON,T0IF		;petekl TMR0?
	return					;  ne
	bcf 	INTCON,T0IF		;nuluj pznak peteen
	decfsz	svtmr,f	; poitadlo peteen 61 x 16,384 ms = 999,4 ms
	return			; jet nen nula
; ukldn do pamti
	movf	ncol,w	; kmitoet LB
	movwf	fr4l
	movf	ncoh,w	; kmitoet MB
	movwf	fr4h
	bcf 	fr4f,0	; rozsah L
	btfsc	R10X
	bsf 	fr4f,0	; rozsah H
	call	savef
; klid
	bcf 	F4  	; zru pznak vyvoln z pamti
	return

;--------------------------------------------------------------------------
mem3	; 02h - kmitoet 3
	btfsc	SAVE	; bylo u uloeno?
	return			;  ano, tak u nic nen teba dlat
	bsf 	F3  	;  ne, nastav pznak a pokrauj
	banksel	OPTION_REG
	btfsc 	OPTION_REG,T0CS	;(1)je asova sputn?
	goto	memtmrset			;  nen
	banksel	PORTA
	btfss 	INTCON,T0IF		;petekl TMR0?
	return					;  ne
	bcf 	INTCON,T0IF		;nuluj pznak peteen
	decfsz	svtmr,f	; poitadlo peteen 61 x 16,384 ms = 999,4 ms
	return			; jet nen nula
; ukldn do pamti
	movf	ncol,w	; kmitoet LB
	movwf	fr3l
	movf	ncoh,w	; kmitoet MB
	movwf	fr3h
	bcf 	fr3f,0	; rozsah L
	btfsc	R10X
	bsf 	fr3f,0	; rozsah H
	call	savef
; klid
	bcf 	F3  	; zru pznak vyvoln z pamti
	return

;--------------------------------------------------------------------------
mem2	; 03h - kmitoet 2
	btfsc	SAVE	; bylo u uloeno?
	return			;  ano, tak u nic nen teba dlat
	bsf 	F2  	;  ne, nastav pznak a pokrauj
	banksel	OPTION_REG
	btfsc 	OPTION_REG,T0CS	;(1)je asova sputn?
	goto	memtmrset			;  nen
	banksel	PORTA
	btfss 	INTCON,T0IF		;petekl TMR0?
	return					;  ne
	bcf 	INTCON,T0IF		;nuluj pznak peteen
	decfsz	svtmr,f	; poitadlo peteen 61 x 16,384 ms = 999,4 ms
	return			; jet nen nula
; ukldn do pamti
	movf	ncol,w	; kmitoet LB
	movwf	fr2l
	movf	ncoh,w	; kmitoet MB
	movwf	fr2h
	bcf 	fr2f,0	; rozsah L
	btfsc	R10X
	bsf 	fr2f,0	; rozsah H
	call	savef
; klid
	bcf 	F2  	; zru pznak vyvoln z pamti
	return

;--------------------------------------------------------------------------
mem1	; 04h - kmitoet 1
	btfsc	SAVE	; bylo u uloeno?
	return			;  ano, tak u nic nen teba dlat
	bsf 	F1  	;  ne, nastav pznak a pokrauj
	banksel	OPTION_REG
	btfsc 	OPTION_REG,T0CS	;(1)je asova sputn?
	goto	memtmrset			;  nen
	banksel	PORTA
	btfss 	INTCON,T0IF		;petekl TMR0?
	return					;  ne
	bcf 	INTCON,T0IF		;nuluj pznak peteen
	decfsz	svtmr,f	; poitadlo peteen 61 x 16,384 ms = 999,4 ms
	return			; jet nen nula
; ukldn do pamti
	movf	ncol,w	; kmitoet LB
	movwf	fr1l
	movf	ncoh,w	; kmitoet MB
	movwf	fr1h
	bcf 	fr1f,0	; rozsah L
	btfsc	R10X
	bsf 	fr1f,0	; rozsah H
	call	savef
; klid
	bcf 	F1  	; zru pznak vyvoln z pamti
	return

memtmrset ; asova nen sputn (prvn prchod)
	bcf 	OPTION_REG,T0CS	;(1)spus asova Fosc/4
	banksel	PORTA
	movlw	.61
	movwf	svtmr
	return

;--------------------------------------------------------------------------
sweep	; 05h - funkce SWEEP
	btfsc	SWEEP		; funkce zvolena?
	return 				; ano
;ne - call recall f3/f4
	movlw	0x80		; zachovej nastaven rozsahu
	andwf	ledst,f
	bsf 	SWEEP		;rozsvi LED f5
	return
;--------------------------------------------------------------------------
finet	; 06h - jemn ladn mezi f3 a F4
	btfsc	FINE		; funkce zvolena?
	return 				; ano
;ne - call recall f3/f4
	movlw	0x80		; zachovej nastaven rozsahu
	andwf	ledst,f
	bsf 	FINE		;rozsvi LED f6
	return
;--------------------------------------------------------------------------
manual	; 07h - ladn v celm rozsahu
	btfsc	MANU		; funkce zvolena?
	return 				; ano
	clrf	ledst
	bsf 	MANU		;rozsvi LED6
; obnov nastaven rozsahu podle pznaku MANR
	btfsc	MANR		; je zvolen vy rozsah u runho ladn?
	bsf 	R10X		;  ano - nastav pznak - rozsvi LED7
	return

;--------------------------------------------------------------------------
hilo	; 08h - rozsah 65/655 kHz, toggle
	movf	kbtmr,f		; nastav pznak ZERO
	btfss	STATUS,Z	; je asova 0?
	goto	hitmrset	;  ne
	btfsc	R10X		; funkce zvolena?
	goto	hilooff		; ano
	bsf 	R10X		; set ON - rozsvi LED8
	btfsc	MANU		; je zvoleno run ladn? ano - nastav pznak MANR
	bsf 	MANR		;  ano - nastav pznak rozsahu pro run ladn
	goto	hitmrset	;  ne
hilooff
	bcf 	R10X		; reset ON - zhasni LED8
	btfsc	MANU		; je zvoleno run ladn?
	bcf 	MANR		;  ano - zru pznak rozsahu pro jemn run ladn
hitmrset
	movlw	.100		; nastav asova na maximum
	movwf	kbtmr
	return

;--------------------------------------------------------------------------
outlvl	; 09h - men vstupnho napt
	bsf 	LVLM	; pznak men napt - vstupn rovn
	clrf	ditmr
	return

;--------------------------------------------------------------------------
coarse	; hrub ladn
	bsf 	flags,1		; nastav pznak hrub ladn
	return

;--------------------------------------------------------------------------
nosw	; 00h - dn tlatko
	bcf 	flags,1		; nuluj pznak hrub ladn
;oeten zkmit tlatek
	movf	kbtmr,f		; nastav pznak ZERO
	btfss	STATUS,Z	; je asova 0?
	decf	kbtmr,f		;  ne
;oeten ukldn do pamti F1 a F4
	banksel	OPTION_REG
	bsf 	OPTION_REG,T0CS	;(1)zastav asova pipojenm vstupu na RA2
	banksel	PORTA
	bcf 	INTCON,T0IF		;nuluj TMR0IF
	clrf	TMR0			;nuluj TMR0
;oeten vyvoln F1 a F4 z pamti
	btfsc	SAVE		; ukldalo se do pamti?
	goto	noswend		;  ano, u jen sma pznaky
	btfss	F1  		; vyvolat F1?
	goto	noswf2		;  ne - jdi dl
; recall F1
	movf	fr1l,w
	movwf	ncol
	movf	fr1h,w
	movwf	ncoh
	clrf	ledst
	btfsc	fr1f,0		; rozsah HI?
	bsf 	ledst,7		;  ano
	bsf 	ledst,3
		; rozsvi LED F1
noswf2
	btfss	F2  		; vyvolat F2?
	goto	noswf3		;  ne - jdi dl
; recall F2
	movf	fr2l,w
	movwf	ncol
	movf	fr2h,w
	movwf	ncoh
	clrf	ledst
	btfsc	fr2f,0		; rozsah HI?
	bsf 	ledst,7		;  ano
	bsf 	ledst,2		; rozsvi LED F2
noswf3
	btfss	F3  		; vyvolat F3?
	goto	noswf4		;  ne - jdi dl
; recall F3
	movf	fr3l,w
	movwf	ncol
	movf	fr3h,w
	movwf	ncoh
	clrf	ledst
	btfsc	fr3f,0		; rozsah HI?
	bsf 	ledst,7		;  ano
	bsf 	ledst,1		; rozsvi LED F3
noswf4
	btfss	F4  		; vyvolat F4?
	goto	noswend		;  ne - jdi dl
;recall F4
	movf	fr4l,w
	movwf	ncol
	movf	fr4h,w
	movwf	ncoh
	clrf	ledst
	btfsc	fr4f,0		; rozsah HI?
	bsf 	ledst,7		;  ano
	bsf 	ledst,0		; rozsvi LED F4
noswend
	movlw	0x07
	andwf	flags,f		; vyma pznaky tlatek F1 a F4 a SAVE
	return

;--------------------------------------------------------------------------
disprange	; nastav rozsah ncoh:ncol x N -> bin2:bin1:bin0, kde N = 1 nebo 10

	movlw	.1
	btfsc	R10X		; rozsah x 10
	movlw	.10

; 16bit by 8bit unsigned multiply by Martin Sturm 2010
; http://www.piclist.com/techref/microchip/math/mul/m16x8mds2.htm
; vstup - W, ncoh:ncol (nemn se)
; vstup - bin2:bin1:bin0
;
;  W x ncoh:ncol -> bin2:bin1:bin0
; 19 cycles, 85-125, 105 average
	clrf	bin2
	clrf	bin1
	clrf	bin0
	movwf	tmp 	; nsobitel
	bsf 	bin0,7
mult1
	rrf 	tmp,f
	btfss	STATUS,C
	goto	mult2
	movfw	ncol
	addwf	bin1,f
	movf	ncoh,w
	btfsc	STATUS,C	;SKPNC
	incfsz	ncoh,w
	addwf	bin2,f
mult2
	rrf 	bin2,f
	rrf 	bin1,f
	rrf 	bin0,f
	btfss	STATUS,C	;SKPC
	goto	mult1
	return
;--------------------------------------------------------------------------
savef ; ulo F1 a F4 z pamti do FLASH a ppne
	bsf 	SAVE	; nastav pznak uloeno
; SAVE FLASH dodlat
	call	heferase	; smae 1 blok HEF od adresy 1f80h
;	call	hefwrite	; zape 16 byte RAM od adresy 40h do HEF od adresy 1f80h
						; nevol se - navazuje za heferase !!
beep
	clrf	tmp
	clrf	cycles
	bsf 	BEEP
	decfsz	tmp,f
	goto	$-1
	bcf 	BEEP
	decfsz	tmp,f
	goto	$-1
	decfsz	cycles,f
	goto	$-7
	return
;--------------------------------------------------------------------------
range	; nastav dli podle rozsahu
	btfss	R10X	; zvolen vy rozsah?
	goto	ranlo	;  ne
	btfsc	R10F	; nastaven vy rozsah?
	return			;  ano - nic nedlej
	; nastav dli na 1:1 a pznak, e je nastaveno
setnodivide
	banksel	NCO1CON
	bcf 	NCO1CON,7	; disable NCO
 ;CLC1 configuration (prchod signlu z CLC2IN0 beze zmny na vstup)
	banksel	CLCDATA
	movlw	0x02
	movwf	CLC1POL
	clrf	CLC1SEL0
	movlw	0x01
	movwf	CLC1SEL1
	movlw	0x20
	movwf	CLC1GLS0
	clrf	CLC1GLS1
	clrf	CLC1GLS2
	clrf	CLC1GLS3
	movlw	0x80
	movwf	CLC1CON
;CLC2 configuration (prchod beze zmny, vstup RC3 (CLC2IN0), vstup RC0 (CLC2)
;	banksel	CLCDATA
	movlw	0x02
	movwf	CLC2POL
	clrf	CLC2SEL0
	clrf	CLC2SEL1
	movlw	0x02
	movwf	CLC2GLS0
	clrf	CLC2GLS1
	clrf	CLC2GLS2
	clrf	CLC2GLS3
	movlw	0x80
	movwf	CLC2CON
;
	banksel	NCO1CON
	bsf 	NCO1CON,7	; enable NCO
	banksel	PORTA
	bsf 	R10F
	return
ranlo
	btfss	R10F	; nastaven ni rozsah?
	return			;  ano - nic nedlej
setdivide10 ; nastav dlen 10
	banksel	NCO1CON
	bcf 	NCO1CON,7	; disable NCO
;CLC1 configuration (dlika 5 spolu s CLC3 a CLC4), nastaven RESET
	banksel	CLCDATA
	movlw	0x04
	movwf	CLC1POL
	movlw	0x74
	movwf	CLC1SEL0
	movlw	0x01
	movwf	CLC1SEL1
	movlw	0x20
	movwf	CLC1GLS0
	movlw	0x08
	movwf	CLC1GLS1
	clrf	CLC1GLS2
	movwf	CLC1GLS3	 ; ulo 0x08
	movlw	0x86
	movwf	CLC1CON
;CLC2 configuration (dlika 2, vstup RC3 (CLC2IN0), bez vstupu na pin)
;	banksel	CLCDATA
	movlw	0x0a
	movwf	CLC2POL
	clrf	CLC2SEL0
	clrf	CLC2SEL1
	movlw	0x02
	movwf	CLC2GLS0
	clrf	CLC2GLS1
	clrf	CLC2GLS2
	clrf	CLC2GLS3
	movlw	0x86		; 0xc6 - vstup na RC0
	movwf	CLC2CON
;CLC3 configuration (dlika 5 spolu s CLC1 a CLC4)
	banksel	CLCDATA
	movlw	0x0a
	movwf	CLC3POL
	movlw	0x40
	movwf	CLC3SEL0
	movlw	0x01
	movwf	CLC3SEL1
	movlw	0x20
	movwf	CLC3GLS0
	movlw	0x08
	movwf	CLC3GLS1
	clrf	CLC3GLS2
	clrf	CLC3GLS3
	movlw	0x86
	movwf	CLC3CON
;CLC4 configuration (dlika 5 spolu s CLC1 a CLC3)
	banksel	CLCDATA
	movlw	0x08
	movwf	CLC4POL
	movlw	0x60
	movwf	CLC4SEL0
	movlw	0x01
	movwf	CLC4SEL1
	movlw	0x20
	movwf	CLC4GLS0
	movlw	0x08
	movwf	CLC4GLS1
	clrf	CLC4GLS2
	clrf	CLC4GLS3
	movlw	0x86
	movwf	CLC4CON
;
	bcf 	CLC1POL,2	; spust dliku v CLC1 - zru RESET JK klopnho obvodu
	banksel	NCO1CON
	bsf 	NCO1CON,7	; enable NCO
	banksel	PORTA
	bcf 	R10F
	return
;--------------------------------------------------------------------------
setnco
	banksel	NCO1CON
	movf	ncoh,w
	movwf	NCO1INCH
	movf	ncol,w
	movwf	NCO1INCL
;	bsf 	NCO1CON,7	; enable NCO
	banksel	PORTA
	return
;--------------------------------------------------------------------------
divide10
;	movlw	.10
;	movwf	tmp
;***********************************************************
;Unsigned 16 bit by 8 bit divide routine
; based on James Hillman 24 by 8 bit divide
;http://www.piclist.com/techref/microchip/math/div/24by8-jah.htm
;
; vstup: 	dlenec: bin1:bin0 (bin1 vy byte)
;			dlitel: tmp (nebo konstanta, zadv se jen jednou)
; vstup:	podl: bin1:bin0
;			zbytek po dlen: bin3
; pomocn prom.: cycles
; 16 instrukc/216 tcy
;***********************************************************
div16b8u
	clrf	bin3	; zbytek
	movlw	.16
	movwf	cycles
div168loop
	rlf 	bin0,w
	rlf 	bin1,f	;
	rlf 	bin3,f	;
	rlf 	bin0,f	;
;	movf	tmp,w
	movlw	.10
	subwf	bin3,f
	btfsc	STATUS,C
	bsf 	bin0,0
	btfss	bin0,0
	addwf	bin3,f
	decfsz	cycles,f
	goto	div168loop
;	return
;zaokrouhlen tady napevno
	movlw	.5
	subwf	bin3,w
	btfss	STATUS,C
	return
	incf	bin0,f
	btfsc   STATUS,Z	; je 0 -> peteklo to
	incf	bin1,f
	return

;--------------------------------------------------------------------------
removeflag
; porovn ledst a ledxch. pokud se li, zru pznak FISW a pekopruje ledst do ledxch
	movf	ledst,w
	andlw	0x7f
	movwf	tmp
	movf	ledxch,w
	andlw	0x7f
	xorwf	tmp,w
	btfsc   STATUS,Z	; je 0 -> jsou shodn
	return
	bcf 	FISW
	movf	ledst,w
	movwf	ledxch
	return

;--------------------------------------------------------------------------
hefread	;pete 16 byte z HEF od adresy 0x1f80 (ni byte pamovho msta)
		; a ulo je do RAM na adresy 0x40 a 0x4f
	clrf	FSR0H
	movlw	0x40
	movwf	FSR0L	; adresa 40h
	movlw   .16
	movwf	cycles
	banksel	PMADRL
	movlw	0x1f
	movwf	PMADRH
	movlw	0x80
	movwf	PMADRL	; adresa L 80h
	bcf 	PMCON1,CFGS

hr_next
	bsf 	PMCON1,RD
	nop
	nop
	movf	PMDATL,w
	movwi	FSR0++
	incf	PMADRL,f
	decfsz  cycles,f
	goto	hr_next
	banksel PORTA
	return

;--------------------------------------------------------------------------
heferase	; smae 1 blok HEF od adresy 1f80h
	bcf 	INTCON,GIE		;disable interrupts
	banksel	PMADRL
	movlw	0x1f
	movwf	PMADRH
	movlw	0x80
	movwf	PMADRL			; adresa L 80h
	bcf 	PMCON1,CFGS		; Not configuration space
	bsf 	PMCON1,FREE		; Specify an erase operation
	bsf 	PMCON1,WREN		; Enable writes
	movlw	0x55			; Start of required sequence to initiate erase
	movwf	PMCON2
	movlw	0xaa
	movwf	PMCON2
	bsf 	PMCON1,WR		; Set WR bit to begin erase
	nop
	nop
	bcf 	PMCON1,WREN		; Disable writes
;	bsf 	INTCON,GIE		; Enable interrupts (if needed)
;	banksel PORTA
;	return

;--------------------------------------------------------------------------
hefwrite	; zape 16 byte RAM od adresy 40h do HEF od adresy 1f80h
;vypln RAM 50h a 5fh hodnotou ffh
	clrf	FSR0H
	movlw	050h	; prvn byte pole dat
	movwf	FSR0L
	movlw	.16		; poet prvk pole
	movwf	cycles	; new
	movlw	0xff	; vpl
addloop
	movwf	INDF0
	incf	FSR0L,f
	decfsz	cycles,f		; konec?
	goto	addloop		; ne
;
;	bcf 	INTCON,GIE		; Disable ints so required sequences will execute properly
							; peruen se nepouv - nen teba ho zakazovat a povolovat
;	banksel	PMADRH			; Bank 3
	movlw	0x1f
	movwf	PMADRH
	movlw	0x80
	movwf	PMADRL			; adresa L 80h

	clrf	FSR0H
	movlw	0x40
	movwf	FSR0L	; adresa 40h

	bcf 	PMCON1,CFGS		; Not configuration space
	bsf 	PMCON1,WREN		; Enable writes
	bsf 	PMCON1,LWLO		; Only Load Write Latches
LOOP
	moviw	FSR0++			; Load first data byte into lower
	movwf	PMDATL
;	moviw	FSR0++			; Load second data byte into upper
	movlw	0x3f		; j do vho bajtu nic nezapisuji
	movwf	PMDATH
	movf	PMADRL,W		; Check if lower bits of address are '00000'
	xorlw	0x1f			; Check if we're on the last of 32 addresses
	andlw	0x1f
	btfsc	STATUS,Z		; Exit if last of 32 words,
	goto	START_WRITE

	movlw	0x55			; Start of required write sequence
	movwf	PMCON2
	movlw	0xaa
	movwf	PMCON2
	bsf 	PMCON1,WR		; Set WR bit to begin write
	nop
	nop

	incf	PMADRL,f		; Still loading latches Increment address
	goto	LOOP			; Write next latches
START_WRITE
	bcf 	PMCON1,LWLO		; No more loading latches - Actually start Flash program memory write
	movlw	0x55			; Start of required write sequence
	movwf	PMCON2
	movlw	0xaa
	movwf	PMCON2
	bsf 	PMCON1,WR		; Set WR bit to begin write
	nop
	nop
	bcf 	PMCON1,WREN		; Disable writes
;	bsf 	INTCON,GIE		; Enable interrupts (if needed)
	banksel PORTA
	return

;--------------------------------------------------------------------------
;delay ; 773 tcy i s call a return (193,25 s pi 16 MHz)
;	movlw	0x00  ;0 DEC
;	movwf	tmp
;	decfsz	tmp,f
;	goto	$-1
;	return

;--------------------------------------------------------------------------
	DE	"Test NCO 13"

	ORG 0x1f80
; F1 = 1000 = 3E8h
	DW	0x3fe8, 0x3f03, 0x3ffe	; nzk kmitoet
; F2 = 30 000 = 7530h
	DW	0x3f30, 0x3f75, 0x3fff	; vysok kmitoet 300 000 = 30 000 * 10
; F3 = 	; 2000 = 7d0h
	DW	0x3fd0, 0x3f07, 0x3ffe	; nzk kmitoet
; F1 = 	; 10 000 = 2710h
	DW	0x3f10, 0x3f27, 0x3ffe	; nzk kmitoet
	DW	0x3fff, 0x3fff, 0x3fff, 0x3f00

    END
